use crate::domain::entity::{Pokemon, PokemonName, PokemonNumber, PokemonTypes};

pub trait Repository: Send + Sync {
    fn insert(&mut self, number: PokemonNumber, name: PokemonName, types: PokemonTypes) -> Insert;
}
#[derive(Debug)]
pub struct InMemoryRepository {
    pokemons: Vec<Pokemon>
}
impl InMemoryRepository {
    pub fn new() -> Self {
        let pokemons: Vec<Pokemon> = vec![];
        Self {
            pokemons
        }
    }
}

pub enum Insert {
    Ok(PokemonNumber),
    Conflict,
}

impl Repository for InMemoryRepository {
    fn insert(&mut self, number: PokemonNumber, name: PokemonName, types: PokemonTypes) -> Insert {
        println!("{number:?} {:?}", self.pokemons);
        if self.pokemons.iter().any(|pokemon| pokemon.number == number) {
            return Insert::Conflict;
        }
        let number_clone = number.clone();
        self.pokemons.push(Pokemon::new(number_clone, name, types));
        println!("{number:?} {:?}", self.pokemons);
        Insert::Ok(number)
    }
}